@@ -16,7 +16,7 @@ class KeySet:
         obj = self.as_dict(is_private, **params)
         return json_dumps(obj)
 
-    def find_by_kid(self, kid):
+    def find_by_kid(self, kid, **params):
         """Find the key matches the given kid value.
 
         :param kid: A string of kid
@@ -27,7 +27,28 @@ class KeySet:
         # of the set if no kid is specified
         if kid is None and len(self.keys) == 1:
             return self.keys[0]
-        for k in self.keys:
-            if k.kid == kid:
-                return k
+
+        keys = [key for key in self.keys if key.kid == kid]
+        if params:
+            keys = list(_filter_keys_by_params(keys, **params))
+
+        if len(keys) == 1:
+            return keys[0]
+
         raise ValueError("Invalid JSON Web Key Set")
+
+
+def _filter_keys_by_params(keys, **params):
+    _use = params.get("use")
+    _alg = params.get("alg")
+
+    for key in keys:
+        designed_use = key.tokens.get("use")
+        if designed_use and _use and designed_use != _use:
+            continue
+
+        designed_alg = key.tokens.get("alg")
+        if designed_alg and _alg and designed_alg != _alg:
+            continue
+
+        yield key
